/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.placeholder.impl;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Properties;

import org.tinygroup.context.Context;
import org.tinygroup.context.impl.ContextImpl;
import org.tinygroup.format.Formater;
import org.tinygroup.format.impl.FormaterImpl;
import org.tinygroup.logger.Logger;
import org.tinygroup.logger.LoggerFactory;
import org.tinygroup.placeholder.FileWriter;
import org.tinygroup.placeholder.PlaceholderProcessor;
import org.tinygroup.placeholder.Validator;
import org.tinygroup.placeholder.config.Validators;
import org.tinygroup.placeholder.filewriter.CommonFileWriter;
import org.tinygroup.placeholder.filewriter.JarFileWriter;
import org.tinygroup.vfs.FileObject;

/**
 * 占位符处理实现
 * 
 * @author luoguo
 * 
 */
public class PlaceholderProcessorImpl implements PlaceholderProcessor {

	private static final String DEPLOY_AREA = "deploy";
	private static final Logger logger = LoggerFactory
			.getLogger(PlaceholderProcessorImpl.class);

	static FileWriter jarFileWriter;
	static FileWriter commonFileWriter;
	static {
		jarFileWriter = new JarFileWriter();
		commonFileWriter = new CommonFileWriter();
	}

	/**
	 * 格式转换环境
	 */
	private Context context = new ContextImpl();
	/**
	 * 格式化对象
	 */
	private transient Formater formater = new FormaterImpl();

	/**
	 * 占位符关键词
	 */
	private String placeholderArea = DEPLOY_AREA;
	/**
	 * 字符集
	 */
	private String encoding = "UTF-8";
	/**
	 * placeholder规则的格式化解析对象
	 */
	private PlaceholderContextFormater contextFormater;

	public String getPlaceholderArea() {
		return placeholderArea;
	}

	public void setPlaceholderArea(String placeholderArea) {
		this.placeholderArea = placeholderArea;
		formater.addFormatProvider(DEPLOY_AREA, contextFormater);
	}

	public String getEncoding() {
		return encoding;
	}

	public void setEncoding(String encoding) {
		this.encoding = encoding;
	}

	public PlaceholderContextFormater getContextFormater() {
		return contextFormater;
	}

	public void setContextFormater(PlaceholderContextFormater contextFormater) {
		this.contextFormater = contextFormater;
	}

	/**
	 * 载入属性文件
	 */
	public void addProperties(FileObject fileObject) {
		Properties properties = new Properties();
		try {
			properties.load(fileObject.getInputStream());
		} catch (Exception e) {
			logger.errorMessage("加载占位符属性文件：[{}]出错", e, fileObject.getAbsolutePath());
		}
		initContext(properties);
	}

	private void initContext(Properties properties) {

		for (Object keyName : properties.keySet()) {
			context.put(keyName.toString(), properties.get(keyName));
		}
	}

	public String processString(String string) {
		return formater.format(context, string);
	}

	public void processFileObject(FileObject fileObject) {
		String content = readContent(fileObject);
		//setPlaceholderArea("deploy");  //通过bean文件注入
		String newContent = processString(content);
		if (!newContent.equals(content)) {
			writeContent(fileObject, newContent);
		} else {
//			try {
//				VFS.getManager().closeFileSystem(fileObject.getFileSystem());
//			} catch (FileSystemException e) {
//				logger.errorMessage(e.getMessage(), e);
//			}
		}
	}

	private void writeContent(FileObject fileObject, String content) {

		String fileName = fileObject.getAbsolutePath();
		if (fileName.indexOf("jar!") != -1) {
			jarFileWriter.writeFile(fileObject, content, encoding);
		} else {
			commonFileWriter.writeFile(fileObject, content, encoding);
		}

		// OutputStream outputStream = null;
		// BufferedWriter write = null;
		// BufferedReader reader = null;
		// try {
		// outputStream = fileObject.getContent().getOutputStream();
		// write = new BufferedWriter(new OutputStreamWriter(outputStream,
		// encoding));
		// reader = new BufferedReader(new StringReader(content));
		// String lines = null;
		// while ((lines = reader.readLine()) != null) {
		// write.write(lines);
		// write.newLine();
		// }
		// } catch (FileSystemException e) {
		// logger.errorMessage(e.getMessage(), e);
		// } catch (IOException e) {
		// logger.errorMessage(e.getMessage(), e);
		// } finally {
		// try {
		//
		// if (write != null) {
		// write.close();
		// }
		// if (outputStream != null) {
		// outputStream.close();
		// }
		// if (reader != null) {
		// reader.close();
		// }
		// } catch (IOException e) {
		// logger.errorMessage("关闭流出错", e);
		// }
		// }
	}

	private String readContent(FileObject fileObject) {
		InputStream inputStream = null;
		BufferedReader reader = null;
		StringBuffer buffer = new StringBuffer();
		try {
			inputStream = fileObject.getInputStream();
			reader = new BufferedReader(new InputStreamReader(inputStream,
					encoding));
			String lines = null;
			while ((lines = reader.readLine()) != null) {
				buffer.append(lines);
			}
		} catch (IOException e) {
			logger.errorMessage(e.getMessage(), e);
		} finally {
			try {
				if (inputStream != null) {
					inputStream.close();
				}
				if (reader != null) {
					reader.close();
				}
			} catch (IOException e) {
				logger.errorMessage("关闭流出错", e);
			}
		}
		return buffer.toString();
	}

	public void addValidators(Validators validators) {
		for (Validator validator : validators.getValidatorList()) {
			contextFormater.addValidator(validator.getName(), validator);
		}

	}
}
